<?php
/** miniOrange enables user to log in using LDAP credentials.
Copyright (C) 2015  miniOrange

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
 * @package 		miniOrange OAuth
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 */
/**
This class contains all the utility functions

 **/
class Mo_Ldap_Util{

    public static $bCode = "RG8gaXQgeW91cnNlbGYgUGxhbiAtIFdQIExEQVAgQ2xvdWQ=";
    public static $pCode = "UHJlbWl1bSBQbGFuIC0gV1AgTERBUCBDbG91ZA==";

    public static function is_customer_registered() {
        $email 			= get_option('mo_ldap_admin_email');
        $customerKey 	= get_option('mo_ldap_admin_customer_key');

        if( ! $email || ! $customerKey || ! is_numeric( trim( $customerKey ) ) ) {
            return 0;
        } else {
            return 1;
        }
    }

    public static function check_empty_or_null( $value ) {
        if( ! isset( $value ) || empty( $value ) ) {
            return true;
        }
        return false;
    }

    public static function encrypt($str){
        if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
            return;
        }

        $key = get_option('mo_ldap_customer_token');
        $method = 'AES-128-ECB';
        $ivSize = openssl_cipher_iv_length($method);
        $iv     = openssl_random_pseudo_bytes($ivSize);
        $strCrypt = openssl_encrypt ($str, $method, $key,OPENSSL_RAW_DATA||OPENSSL_ZERO_PADDING, $iv);
        return base64_encode($iv.$strCrypt);
    }

    public static function decrypt($value)
    {
        if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
            return;
        }

        $strIn = base64_decode($value);
        $key = get_option('mo_ldap_customer_token');
        $method = 'AES-128-ECB';
        $ivSize = openssl_cipher_iv_length($method);
        $iv     = substr($strIn,0,$ivSize);
        $data   = substr($strIn,$ivSize);
        $clear  = openssl_decrypt ($data, $method, $key, OPENSSL_RAW_DATA||OPENSSL_ZERO_PADDING, $iv);

        return $clear;
    }

    public static function is_curl_installed() {
        if  (in_array  ('curl', get_loaded_extensions())) {
            return 1;
        } else
            return 0;
    }

    public static function is_extension_installed($name) {
        if  (in_array  ($name, get_loaded_extensions())) {
            return true;
        }
        else {
            return false;
        }
    }

    public static function mo_ldap_is_customer_validated(){
        $email 			= get_option('mo_ldap_admin_email');
        $customerKey 	= get_option('mo_ldap_admin_customer_key');
        if( ! $email || ! $customerKey || ! is_numeric( trim( $customerKey ) ) ) {
            return 0;
        } else {
            return get_option('mo_ldap_check_ln') ? get_option('mo_ldap_check_ln') :  0;
        }
    }

    function check_customer_ln(){

        $url = get_option('mo_ldap_host_name') . '/moas/rest/customer/license';
        $ch = curl_init($url);

        /* The customer Key provided to you */
        $customerKey = get_option('mo_ldap_admin_customer_key');

        /* The customer API Key provided to you */
        $apiKey = get_option('mo_ldap_admin_api_key');

        /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
        $currentTimeInMillis = round(microtime(true) * 1000);

        /* Creating the Hash using SHA-512 algorithm */
        $stringToHash = $customerKey . number_format($currentTimeInMillis, 0, '', '') . $apiKey;
        $hashValue = hash("sha512", $stringToHash);

        $customerKeyHeader = "Customer-Key: " . $customerKey;
        $timestampHeader = "Timestamp: " . $currentTimeInMillis;
        $authorizationHeader = "Authorization: " . $hashValue;

        $fields = '';

        //*check for otp over sms/email
        $fields = array(
            'customerId' => $customerKey,
            'applicationName' => 'wp_ldap_cloud'
        );

        $field_string = json_encode($fields);

        curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
        curl_setopt( $ch, CURLOPT_ENCODING, "" );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );    # required for https urls

        curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", $customerKeyHeader,
            $timestampHeader, $authorizationHeader));
        curl_setopt( $ch, CURLOPT_POST, true);
        curl_setopt( $ch, CURLOPT_POSTFIELDS, $field_string);
        curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt( $ch, CURLOPT_TIMEOUT, 20);
        $content = curl_exec($ch);

        if(curl_errno($ch)){
            return null;
        }
        curl_close($ch);
        return $content;
    }
}
?>