<?php 
    /*
    Plugin Name: Active Directory Integration for Cloud
    Plugin URI: http://miniorange.com
    Description: Plugin for login into Wordpress through credentials stored in LDAP
    Author: miniorange
    Version: 3.7
    Author URI: http://miniorange.com
    */
	
	require_once 'generate_saml_assertion.php';
	require_once 'mo_ldap_pages.php';
	require('mo_ldap_support.php');
	require('class-mo-ldap-customer-setup.php');
	require('class-mo-ldap-utility.php');
	require('class-mo-ldap-config.php');
	require('class-mo-ldap-role-mapping.php');
	
	class Mo_Ldap_Login{
		
		function __construct(){
			add_action('admin_menu', array($this, 'mo_ldap_login_widget_menu'));
			add_action('admin_init', array($this, 'login_widget_save_options'));
			add_action('init', array($this, 'test_attribute_configuration'));
			add_action( 'admin_enqueue_scripts', array( $this, 'mo_ldap_settings_style' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'mo_ldap_settings_script' ) );
			add_action('parse_request', array($this, 'parse_sso_request'));
			add_action('show_user_profile', array($this, 'show_user_profile'));
			add_action('profile_update', array($this, 'update_profile'), 10, 2);
			remove_action( 'admin_notices', array( $this, 'success_message') );
			remove_action( 'admin_notices', array( $this, 'error_message') );
			add_filter('query_vars', array($this, 'plugin_query_vars'));
			register_deactivation_hook(__FILE__, array( $this, 'mo_ldap_deactivate'));
			if(get_option('mo_ldap_enable_login') == 1){
				remove_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
				add_filter('authenticate', array($this, 'ldap_login'), 100, 3);
                add_filter( 'gettext', array($this, 'remove_lost_password_link'), 20, 3);
			}
			register_activation_hook( __FILE__, array($this,'mo_ldap_activate')) ;
		}
        
        function remove_lost_password_link($translated_text, $text, $domain){
            return str_replace( array('Lost your password?', 'Lost your password'), '', trim($text, '?') );
        }

		
		function ldap_login($user, $username, $password){
			if(empty($username) || empty ($password)){        
				//create new error object and add errors to it.
				$error = new WP_Error();

				if(empty($username)){ //No email
					$error->add('empty_username', __('<strong>ERROR</strong>: Email field is empty.'));
				}
				
				if(empty($password)){ //No password
					$error->add('empty_password', __('<strong>ERROR</strong>: Password field is empty.'));
				}
				return $error;
			}
		
			if(get_option('mo_ldap_enable_both_login')){
				if( username_exists( $username)){
					$user = get_user_by("login",$username);
					if ( $user && wp_check_password( $password, $user->data->user_pass, $user->ID) )
						return $user;					
				}
			}
			
			$mo_ldap_config = new Mo_Ldap_Config();
			$decoded_response = $mo_ldap_config->ldap_login($username, $password);
			$status = $decoded_response['statusCode'];
			
			if($status == 'SUCCESS'){
			  if( username_exists( $username)) {
				  $user = get_userdatabylogin($username);
					
				  //Role Mapping
				  if(get_option('mo_ldap_enable_role_mapping')) {
				  	$mo_ldap_role_mapping = new Mo_Ldap_Role_Mapping();
				  	$member_of_attr_array = $mo_ldap_role_mapping->get_member_of_attribute($username);
				  	$mo_ldap_role_mapping->mo_ldap_update_role_mapping($user->ID,$member_of_attr_array);
				  }
				  
				  //Attribute Mapping
				  if(get_option('mo_ldap_enable_attribute_mapping')) {
					 // print_r($decoded_response);
					 if(isset($decoded_response ['profileAttributes']))
						$this->update_user_attributes($decoded_response ['profileAttributes'],$user->ID);
				  } 
				  
				  return $user;
			   } else {
				   
					   if(!get_option('mo_ldap_register_user')) {
							$error = new WP_Error();
							$error->add('registration_disabled_error', __('<strong>ERROR</strong>: Your Administrator has not enabled Auto Registration. Please contact your Administrator.'));
							return $error;
						}else{
							//create user if not exists
						   $random_password 	= wp_generate_password( 10, false );
						   $userdata = array(
								'user_login'  =>  $username,
								'user_pass'   =>  $random_password  // When creating an user, `user_pass` is expected.
							);
						
							$user_id = wp_insert_user( $userdata ) ;

							//On success
							if( !is_wp_error($user_id) ) {
								$user = get_userdatabylogin($username);
								
								//Role Mapping
								if(get_option('mo_ldap_enable_role_mapping')) {
									$mo_ldap_role_mapping = new Mo_Ldap_Role_Mapping();
									$member_of_attr_array = $mo_ldap_role_mapping->get_member_of_attribute($username);
									$mo_ldap_role_mapping->mo_ldap_update_role_mapping($user->ID,$member_of_attr_array);
								}
								
								//Attribute Mapping
								if(get_option('mo_ldap_enable_attribute_mapping')) {
									 // print_r($decoded_response);
									if(isset($decoded_response ['profileAttributes']))
										$this->update_user_attributes($decoded_response ['profileAttributes'],$user->ID);
								}
								
								return $user;
							}else{
								$error = new WP_Error();
								$error->add('registration_error', __('<strong>ERROR</strong>: There was an error registering your account. Please try again.'));
								return $error;
							}
					}
				}
				wp_redirect( site_url() );
				exit;
						
			} else if($status == 'CURL_ERROR'){
				$error = new WP_Error();
				$error->add('curl_error', __('<strong>ERROR</strong>: <a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.'));
				return $error;
			} else if($status == 'OPENSSL_ERROR'){
                $error = new WP_Error();
                $error->add('OPENSSL_ERROR', __('<strong>ERROR</strong>: <a target="_blank" href="http://php.net/manual/en/openssl.installation.php">PHP OpenSSL extension</a> is not installed or disabled.'));
                return $error;
			} else {
				$error = new WP_Error();
				$error->add('incorrect_credentials', __('<strong>ERROR</strong>: Invalid username or incorrect password. Please try again.'));
				return $error;
			}
		}
	
		function mo_ldap_login_widget_menu(){
			add_menu_page ('LDAP/AD Login for Cloud', 'LDAP/AD Login for Cloud', 'activate_plugins', 'mo_ldap_login', array( $this, 'mo_ldap_login_widget_options'),plugin_dir_url(__FILE__) . 'includes/images/miniorange_icon.png');
		}
		
		
		function update_profile($user_id, $old_user_data){
			$custom_attributes = array();
			foreach($_POST as $post_param=>$value){
				if(strpos($post_param, "mo_ldap_custom_attribute") === false){
					//Do nothing
				}else{
					$custom_attributes[str_replace('mo_ldap_custom_attribute_', '', $post_param)] = $value;
				}
			}
			foreach($custom_attributes as $attribute=>$value){
				update_user_meta($user_id, 'mo_ldap_custom_attribute_' . $attribute, $value);
			}		
		}
		
		function show_user_profile($user){

			$custom_attributes = array();
			$wp_options = wp_load_alloptions();

			?>
			<h3>Extra profile information</h3>
				<table class="form-table">
					<?php
					foreach($wp_options as $option=>$value){
						if(strpos($option, "mo_ldap_custom_attribute") === false){
							//Do nothing
						}else{
							?>
							<tr>
								<td><b><font color="#FF0000"></font><?php echo $value; ?></b></td>
								<td><input type="text" name="<?php echo $option; ?>" value="<?php echo get_user_meta($user->ID, $option, true); ?>" style="width:61%;" /></td>
							</tr><?php
						}
					}
				?>
				</table>
			
			<?php
		}

		
		function mo_ldap_login_widget_options(){
			$xecurify_migration = get_option('mo_ldap_xecurify_migration');
			if($xecurify_migration == 'true')
				update_option( 'mo_ldap_host_name', 'https://login.xecurify.com' );
			else
				update_option( 'mo_ldap_host_name', 'https://auth.miniorange.com' );
			//Setting default configuration
			$default_config = array(
				'server_url' => 'ldap://58.64.132.235:389',
				'service_account_dn' => 'cn=testuser,cn=Users,dc=miniorange,dc=com',
				'admin_password' => 'XXXXXXXX',
				'dn_attribute' => 'distinguishedName',
				'search_base' => 'cn=Users,dc=miniorange,dc=com',
				'search_filter' => '(&(objectClass=*)(cn=?))',
				'test_username' => 'testuser',
				'test_password' => 'password'
			);
			update_option( 'mo_ldap_default_config', $default_config );
			mo_ldap_settings();
		}
		
		function login_widget_save_options(){
			if(isset($_POST['option'])){
				if($_POST['option'] == "mo_ldap_register_customer") {		//register the customer
				
					//validate and sanitize
					$email = '';
					$phone = '';
					$password = '';
					$confirmPassword = '';
					$fname = '';
					$lname = '';
					$companyName = '';
					if( Mo_Ldap_Util::check_empty_or_null( $_POST['email'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['phone'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['password'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['confirmPassword'] ) || 
						Mo_Ldap_Util::check_empty_or_null($_POST['company']) ) {
						update_option( 'mo_ldap_message', 'All the fields are required. Please enter valid entries.');
						$this->show_error_message();
						return;
					} else if( strlen( $_POST['password'] ) < 6 || strlen( $_POST['confirmPassword'] ) < 6){	//check password is of minimum length 6
						update_option( 'mo_ldap_message', 'Choose a password with minimum length 6.');
						$this->show_error_message();
						return;
					} else{
						$email = sanitize_email( $_POST['email'] );
						$phone = sanitize_text_field( $_POST['phone'] );
						$password = sanitize_text_field( $_POST['password'] );
						$confirmPassword = sanitize_text_field( $_POST['confirmPassword'] );
						$fname = sanitize_text_field( $_POST['fname']);
						$lname = sanitize_text_field( $_POST['lname']);
						$companyName = sanitize_text_field( $_POST['company']);
					}
					update_option( 'mo_ldap_admin_email', $email );
					update_option( 'mo_ldap_admin_phone', $phone );
					
					update_option('mo_ldap_admin_fname', $fname);
					update_option('mo_ldap_admin_lname', $lname);
					update_option('mo_ldap_admin_company', $companyName);
					
					if( strcmp( $password, $confirmPassword) == 0 ) {
						update_option( 'mo_ldap_password', $password );

						$customer = new Mo_Ldap_Customer();
						$content = json_decode($customer->check_customer(), true);
						if( strcasecmp( $content['status'], 'CUSTOMER_NOT_FOUND') == 0 ){
							$content = json_decode($customer->send_otp_token(), true);
							if(strcasecmp($content['status'], 'SUCCESS') == 0) {
								update_option( 'mo_ldap_message', ' A one time passcode is sent to ' . get_option('mo_ldap_admin_email') . '. Please enter the otp here to verify your email.');
								update_option('mo_ldap_transactionId',$content['txId']);
								update_option('mo_ldap_registration_status','MO_OTP_DELIVERED_SUCCESS');

								$this->show_success_message();
							} else {
								update_option('mo_ldap_message','There was an error in sending email. Please click on Resend OTP to try again.');
								update_option('mo_ldap_registration_status','MO_OTP_DELIVERED_FAILURE');
								$this->show_error_message();
							}
						} else if( strcasecmp( $content['status'], 'CURL_ERROR') == 0 ){
							update_option('mo_ldap_message', $content['statusMessage']);
							update_option('mo_ldap_registration_status','MO_OTP_DELIVERED_FAILURE');
							$this->show_error_message();
						} else{
							$content = $customer->get_customer_key();
							$customerKey = json_decode($content, true);
							if(json_last_error() == JSON_ERROR_NONE) {
								$this->save_success_customer_config($customerKey['id'], $customerKey['apiKey'], $customerKey['token'], 'Your account has been retrieved successfully.');
								update_option('mo_ldap_password', '');
							} else {
								update_option( 'mo_ldap_message', 'You already have an account with miniOrange. Please enter a valid password.');
								update_option('mo_ldap_verify_customer', 'true');
								delete_option('mo_ldap_new_registration');
								$this->show_error_message();
							}
						}

					} else {
						update_option( 'mo_ldap_message', 'Password and Confirm password do not match.');
						delete_option('mo_ldap_verify_customer');
						$this->show_error_message();
					}
				} 
				else if( $_POST['option'] == "mo_ldap_verify_customer" ) {	//login the admin to miniOrange
			
					//validation and sanitization
					$email = '';
					$password = '';
					if( Mo_Ldap_Util::check_empty_or_null( $_POST['email'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['password'] ) ) {
						update_option( 'mo_ldap_message', 'All the fields are required. Please enter valid entries.');
						$this->show_error_message();
						return;
					} else{
						$email = sanitize_email( $_POST['email'] );
						$password = sanitize_text_field( $_POST['password'] );
					}
				
					update_option( 'mo_ldap_admin_email', $email );
					update_option( 'mo_ldap_password', $password );
					$customer = new Mo_Ldap_Customer();
					$content = $customer->get_customer_key();
					$customerKey = json_decode( $content, true );
					if( strcasecmp( $customerKey['apiKey'], 'CURL_ERROR') == 0) {
						update_option('mo_ldap_message', $customerKey['token']);
						$this->show_error_message();
					} else if( json_last_error() == JSON_ERROR_NONE ) {
						update_option( 'mo_ldap_admin_phone', $customerKey['phone'] );
						$this->save_success_customer_config($customerKey['id'], $customerKey['apiKey'], $customerKey['token'], 'Your account has been retrieved successfully.');
						update_option('mo_ldap_password', '');
					} else {
						update_option( 'mo_ldap_message', 'Invalid username or password. Please try again.');
						$this->show_error_message();		
					}
					update_option('mo_ldap_password', '');
				}
				else if( $_POST['option'] == "mo_ldap_enable" ) {		//enable ldap login
					update_option( 'mo_ldap_enable_login', isset($_POST['enable_ldap_login']) ? $_POST['enable_ldap_login'] : 0);
					if(get_option('mo_ldap_enable_login')) {
						update_option( 'mo_ldap_message', 'Login through your LDAP has been enabled.');
						$this->show_success_message();
					} else {
						update_option( 'mo_ldap_message', 'Login through your LDAP has been disabled.');
						$this->show_success_message();
					}
				}
				else if( $_POST['option'] == "mo_ldap_register_user" ) {		//enable auto registration of users
					update_option( 'mo_ldap_register_user', isset($_POST['mo_ldap_register_user']) ? $_POST['mo_ldap_register_user'] : 0);
					if(get_option('mo_ldap_register_user')) {
						update_option( 'mo_ldap_message', 'Auto Registering users has been enabled.');
						$this->show_success_message();
					} else {
						update_option( 'mo_ldap_message', 'Auto Registering users has been disabled.');
						$this->show_success_message();
					}
				}
				else if( $_POST['option'] == "mo_ldap_save_config" ) {		//save ldap configuration
					
					//validation and sanitization
					$server_name = '';
					$dn = '';
					$admin_ldap_password = '';
					$dn_attribute = '';
					$search_base = '';
					$search_filter = "(&(objectClass=*)(cn=?))";
					if( Mo_Ldap_Util::check_empty_or_null( $_POST['ldap_server'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['dn'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['admin_password'] ) ||  Mo_Ldap_Util::check_empty_or_null( $_POST['search_base'] ) ) {
						update_option( 'mo_ldap_message', 'All the fields are required. Please enter valid entries.');
						$this->show_error_message();
						return;
					} else{
						$server_name = sanitize_text_field( $_POST['ldap_server'] );
						$dn = sanitize_text_field( $_POST['dn'] );
						$admin_ldap_password = sanitize_text_field( $_POST['admin_password'] );
						$search_base = sanitize_text_field( $_POST['search_base'] );
						if(get_option('mo_ldap_free_version')){
							if(strpos($search_base, ';')){
								$pricing_url = add_query_arg( array("tab" => "pricing"), $_SERVER["REQUEST_URI"] );
								$message = 'You have entered multiple search bases. Multiple Search Bases are supported in the <b>Premium version</b> of the plugin. <a href="' . $pricing_url .'">Click here to upgrade</a> ';
								update_option( 'mo_ldap_message', $message);
								$this->show_error_message();
								return;
							}
						}
						if(isset($_POST['search_filter'])){
							if(!Mo_Ldap_Util::check_empty_or_null($_POST['search_filter'])){
								$search_filter = sanitize_text_field( $_POST['search_filter'] );
							}
						}
					}
					
					//Encrypting all fields and storing them
					if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
						update_option( 'mo_ldap_message', 'PHP openssl extension is not installed or disabled. Please enable it first.');
						$this->show_error_message();
					}else{
						update_option( 'mo_ldap_server_url', Mo_Ldap_Util::encrypt($server_name));
						update_option( 'mo_ldap_server_dn', Mo_Ldap_Util::encrypt($dn));
						update_option( 'mo_ldap_server_password', Mo_Ldap_Util::encrypt($admin_ldap_password));
						//update_option( 'mo_ldap_dn_attribute', Mo_Ldap_Util::encrypt($dn_attribute));
						update_option( 'mo_ldap_search_base', Mo_Ldap_Util::encrypt($search_base));
						update_option( 'mo_ldap_search_filter', Mo_Ldap_Util::encrypt($search_filter));
						
						$mo_ldap_config = new Mo_Ldap_Config();
						
						//Save LDAP configuration
						$save_content = $mo_ldap_config->save_ldap_config();
						$save_response = json_decode( $save_content, true );
						$message = '';
						$status = 'error';
						
						if(strcasecmp($save_response['statusCode'], 'SUCCESS') == 0) {
							$message = $message . 'Your configuration has been saved.';
							$status = 'success';
						} else if(strcasecmp($save_response['statusCode'], 'ERROR') == 0) {
							$message = $message . $save_response['statusMessage'];
						} else if( strcasecmp( $save_response['statusCode'], 'CURL_ERROR') == 0) {
							update_option('mo_ldap_message', $save_response['statusMessage']);
							$this->show_error_message();
							return;
						} else {
							$message = $message. 'There was an error in saving your configuration.';
						}
						
						//Test connection with the LDAP configuration provided. This makes a call to check if connection is established successfully.
						$content = $mo_ldap_config->test_connection(null);
						$response = json_decode( $content, true );
						
						if(strcasecmp($response['statusCode'], 'SUCCESS') == 0) {
							if(strcasecmp($status, 'success') == 0) {
								update_option( 'mo_ldap_message', $message . ' Connection was established successfully. Please test authentication to verify LDAP User Mapping Configuration.' );
								$this->show_success_message();
							} else {
								update_option( 'mo_ldap_message', $message . ' Connection was established successfully.');
								$this->show_error_message();
							}
						} else if(strcasecmp($response['statusCode'], 'ERROR') == 0) {
							update_option( 'mo_ldap_message', $message . ' ' . $response['statusMessage'] . ' Please make sure to open your firewall to allow incoming requests to your LDAP from hosts - 52.55.147.107 , 52.86.38.163 and open port 389(636 for SSL or ldaps). Test using Ping LDAP Server.');
							$this->show_error_message();
						} else if( strcasecmp( $response['statusCode'], 'CURL_ERROR') == 0) {
							update_option( 'mo_ldap_message', $message . ' ' . $response['statusMessage']);
							$this->show_error_message();
						} else {
							update_option( 'mo_ldap_message', $message . ' There was an error in connecting with the current settings. Please make sure to open your firewall to allow incoming requests to your LDAP from hosts - 52.55.147.107 , 52.86.38.163 and open port 389(636 for SSL or ldaps). Test using Ping LDAP Server.');
							$this->show_error_message();
						}
					}
				}
				else if( $_POST['option'] == "mo_ldap_test_auth" ) {		//test authentication with current settings
					$server_name = get_option( 'mo_ldap_server_url');
					$dn = get_option( 'mo_ldap_server_dn');
					$admin_ldap_password = get_option( 'mo_ldap_server_password');
					//$dn_attribute = get_option( 'mo_ldap_dn_attribute');
					$search_base = get_option( 'mo_ldap_search_base');
					$search_filter = get_option( 'mo_ldap_search_filter');
					
					//validation and sanitization
					$test_username = '';
					$test_password = '';
					
					//Check if username and password are empty
					if( Mo_Ldap_Util::check_empty_or_null( $_POST['test_username'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['test_password'] ) ) {
						update_option( 'mo_ldap_message', 'All the fields are required. Please enter valid entries.');
						$this->show_error_message();
						return;
					} 
					//Check if configuration is saved
					else if( Mo_Ldap_Util::check_empty_or_null( $server_name ) || Mo_Ldap_Util::check_empty_or_null( $dn ) || Mo_Ldap_Util::check_empty_or_null( $admin_ldap_password ) || Mo_Ldap_Util::check_empty_or_null( $search_base ) || Mo_Ldap_Util::check_empty_or_null( $search_filter ) ) {
						update_option( 'mo_ldap_message', 'Please save LDAP Configuration to test authentication.');
						$this->show_error_message();
						return;
					} else{
						$test_username = sanitize_text_field( $_POST['test_username'] );
						$test_password = sanitize_text_field( $_POST['test_password'] );
					}
					
					//Call to authenticate test
					$mo_ldap_config = new Mo_Ldap_Config();
					$content = $mo_ldap_config->test_authentication($test_username, $test_password, null);
					$response = json_decode( $content, true );
					
					if(strcasecmp($response['statusCode'], 'SUCCESS') == 0) {
						update_option( 'mo_ldap_message', 'Test is successful! Your credentials have matched.');
						$this->show_success_message();
					} else if(strcasecmp($response['statusCode'], 'ERROR') == 0) {
						update_option( 'mo_ldap_message', $response['statusMessage'] . ' Please verify the Search Base(s) and Search filter. Your user should be present in the Search base defined.');
						$this->show_error_message();
					} else if( strcasecmp( $response['statusCode'], 'CURL_ERROR') == 0) {
						update_option('mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else if( strcasecmp( $response['statusCode'], 'OPENSSL_ERROR') == 0) {
						update_option('mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else {
						update_option( 'mo_ldap_message', 'There was an error processing your request. Please verify the Search Base(s) and Search filter. Your user should be present in the Search base defined.');
						$this->show_error_message();
					}		
				}
				else if( $_POST['option'] == "mo_ldap_test_default_auth" ) {		//test default authentication with current settings
					$default_config = get_option('mo_ldap_default_config');
				
					$default_test_username = $default_config['test_username'];
					$default_test_password = $default_config['test_password'];
					
					//Call to test default authentication
					$mo_ldap_config = new Mo_Ldap_Config();
					$content = $mo_ldap_config->test_authentication($default_test_username, $default_test_password, true);
					$response = json_decode( $content, true );
					
					if(strcasecmp($response['statusCode'], 'SUCCESS') == 0) {
						update_option( 'mo_ldap_message', 'Authenticated successfully.');
						$this->show_success_message();
					} else if(strcasecmp($response['statusCode'], 'ERROR') == 0) {
						update_option( 'mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else if( strcasecmp( $response['statusCode'], 'CURL_ERROR') == 0) {
						update_option('mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else if( strcasecmp( $response['statusCode'], 'OPENSSL_ERROR') == 0) {
						update_option('mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else {
						update_option( 'mo_ldap_message', 'There was an error processing your request.');
						$this->show_error_message();
					}
				}
				else if( $_POST['option'] == "mo_ldap_test_default_config" ) {		//test default connection with current settings
					//Call to test connection
					$mo_ldap_config = new Mo_Ldap_Config();
					$content = $mo_ldap_config->test_connection(true);
					$response = json_decode( $content, true );
					
					if(strcasecmp($response['statusCode'], 'SUCCESS') == 0) {
						update_option( 'mo_ldap_message', 'Connection was established successfully.');
						$this->show_success_message();
					} else if(strcasecmp($response['statusCode'], 'ERROR') == 0) {
						update_option( 'mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else if( strcasecmp( $response['statusCode'], 'CURL_ERROR') == 0) {
						update_option('mo_ldap_message', $response['statusMessage']);
						$this->show_error_message();
					} else {
						update_option( 'mo_ldap_message', 'There was an error processing your request.');
						$this->show_error_message();
					}
				}
				else if($_POST['option'] == "mo_ldap_login_send_query"){
					$query = '';
					if( Mo_Ldap_Util::check_empty_or_null( $_POST['query_email'] ) || Mo_Ldap_Util::check_empty_or_null( $_POST['query'] ) ) {
						update_option( 'mo_ldap_message', 'Please submit your query along with email.');
						$this->show_error_message();
						return;
					} else{
						$query = sanitize_text_field( $_POST['query'] );
						$email = sanitize_text_field( $_POST['query_email'] );
						$phone = sanitize_text_field( $_POST['query_phone'] );
						$contact_us = new Mo_Ldap_Customer();
						$submited = json_decode($contact_us->submit_contact_us($email, $phone, $query),true);
						
						if( strcasecmp( $submited['status'], 'CURL_ERROR') == 0) {
							update_option('mo_ldap_message', $submited['statusMessage']);
							$this->show_error_message();
						} else if(json_last_error() == JSON_ERROR_NONE) {
							if ( $submited == false ) {
								update_option('mo_ldap_message', 'Your query could not be submitted. Please try again.');
								$this->show_error_message();
							} else {
								update_option('mo_ldap_message', 'Thanks for getting in touch! We shall get back to you shortly.');
								$this->show_success_message();
							}
						}

					}
				}  else if($_POST['option'] == 'mo_ldap_enable_both_login'){
					update_option( 'mo_ldap_enable_both_login', isset($_POST['mo_ldap_enable_both_login']) ? $_POST['mo_ldap_enable_both_login'] : 0);
					if(get_option('mo_ldap_enable_both_login')) {
						update_option( 'mo_ldap_message', 'Login with both LDAP and WordPress has been enabled.');
						$this->show_success_message();
					} else {
						update_option( 'mo_ldap_message', 'Login with both LDAP and WordPress has been disabled.');
						$this->show_success_message();
					}
				} else if( $_POST['option'] == "mo_ldap_resend_otp" ) {			//send OTP to user to verify email
					$customer = new Mo_Ldap_Customer();
					$content = json_decode($customer->send_otp_token(), true);
					if(strcasecmp($content['status'], 'SUCCESS') == 0) {
							update_option( 'mo_ldap_message', ' A one time passcode is sent to ' . get_option('mo_ldap_admin_email') . ' again. Please enter the OTP recieved.');
							update_option('mo_ldap_transactionId',$content['txId']);
							update_option('mo_ldap_registration_status','MO_OTP_DELIVERED_SUCCESS');
							$this->show_success_message();
					} else if( strcasecmp( $content['status'], 'CURL_ERROR') == 0) {
						update_option('mo_ldap_message', $content['statusMessage']);
						update_option('mo_ldap_registration_status','MO_OTP_DELIVERED_FAILURE');
						$this->show_error_message();
					} else{
							update_option('mo_ldap_message','There was an error in sending email. Please click on Resend OTP to try again.');
							update_option('mo_ldap_registration_status','MO_OTP_DELIVERED_FAILURE');
							$this->show_error_message();
					}
				}
				else if( $_POST['option'] == "mo_ldap_validate_otp"){		//verify OTP entered by user

					//validation and sanitization
					$otp_token = '';
					if( Mo_Ldap_Util::check_empty_or_null( $_POST['otp_token'] ) ) {
						update_option( 'mo_ldap_message', 'Please enter a value in otp field.');
						update_option('mo_ldap_registration_status','MO_OTP_VALIDATION_FAILURE');
						$this->show_error_message();
						return;
					} else{
						$otp_token = sanitize_text_field( $_POST['otp_token'] );
					}

					$customer = new Mo_Ldap_Customer();
					$content = json_decode($customer->validate_otp_token(get_option('mo_ldap_transactionId'), $otp_token ),true);
					if(strcasecmp($content['status'], 'SUCCESS') == 0) {
						$customer = new Mo_Ldap_Customer();
						$customerKey = json_decode($customer->create_customer(), true);
						if(strcasecmp($customerKey['status'], 'CUSTOMER_USERNAME_ALREADY_EXISTS') == 0) {	//admin already exists in miniOrange
							$content = $customer->get_customer_key();
							$customerKey = json_decode($content, true);
							if(json_last_error() == JSON_ERROR_NONE) {
								$this->save_success_customer_config($customerKey['id'], $customerKey['apiKey'], $customerKey['token'], 'Your account has been retrieved successfully.');
							} else {
								update_option( 'mo_ldap_message', 'You already have an account with miniOrange. Please enter a valid password.');
								update_option('mo_ldap_verify_customer', 'true');
								delete_option('mo_ldap_new_registration');
								$this->show_error_message();
							}
						} else if(strcasecmp($customerKey['status'], 'SUCCESS') == 0) { 	//registration successful
							update_option('mo_ldap_search_filter', Mo_Ldap_Util::encrypt("(&(objectClass=*)(cn=?))"));
							update_option( 'mo_ldap_free_version', 1 );
							$this->save_success_customer_config($customerKey['id'], $customerKey['apiKey'], $customerKey['token'], 'Registration complete!');
						}
						update_option('mo_ldap_password', '');
					} else if( strcasecmp( $content['status'], 'CURL_ERROR') == 0) {
						update_option('mo_ldap_message', $content['statusMessage']);
						update_option('mo_ldap_registration_status','MO_OTP_VALIDATION_FAILURE');
						$this->show_error_message();
					} else{
						update_option( 'mo_ldap_message','Invalid one time passcode. Please enter a valid otp.');
						update_option('mo_ldap_registration_status','MO_OTP_VALIDATION_FAILURE');
						$this->show_error_message();
					}
				} else if($_POST['option'] == 'mo_ldap_ping_server'){
					
					//Sanitize form fields
					$ldap_server_url = sanitize_text_field($_POST['ldap_server']);
					$customer = new Mo_Ldap_Customer();
					$content = json_decode($customer->ping_ldap_server($ldap_server_url), true);
					
					if(strcasecmp($content['statusCode'], 'SUCCESS') == 0){
						update_option('mo_ldap_message', 'Successfully contacted LDAP Server');
						$this->show_success_message();
					} else{
						$error_message = $content['statusMessage'];
						update_option('mo_ldap_message', 'Error contacting LDAP Server: ' . $error_message);
						$this->show_error_message();
					}	
				} else if($_POST['option'] == 'mo_ldap_enable_role_mapping'){
					update_option( 'mo_ldap_enable_role_mapping', isset($_POST['enable_ldap_role_mapping']) ? $_POST['enable_ldap_role_mapping'] : 0);
					if(get_option('mo_ldap_enable_role_mapping')) {
						update_option( 'mo_ldap_message', 'LDAP Group to WP role mapping has been enabled.');
						$this->show_success_message();
					} else {
						update_option( 'mo_ldap_message', 'LDAP Group to WP role mapping has been disabled.');
						$this->show_success_message();
					}
				} else if($_POST['option'] == 'mo_ldap_save_role_mapping'){
					$max_allowed_mappings = 100;
					$added_mappings_count = 0 ;
					for($i=1;$i<=$max_allowed_mappings;$i++){
						if(isset($_POST['mapping_key_'.$i]) && isset($_POST['mapping_key_'.$i])){
							if($_POST['mapping_key_'.$i]=="")
								continue;
							update_option( 'mo_ldap_mapping_key_'.$i, $_POST['mapping_key_'.$i]);
							update_option( 'mo_ldap_mapping_value_'.$i, $_POST['mapping_value_'.$i]);
							$added_mappings_count++;
						}else{
							break;
						}
					}
					update_option( 'mo_ldap_role_mapping_count', $added_mappings_count);
					if(isset($_POST['mapping_value_default']))
						update_option('mo_ldap_mapping_value_default', $_POST['mapping_value_default']);
					if(isset($_POST['mapping_memberof_attribute']))
						update_option('mo_ldap_mapping_memberof_attribute', $_POST['mapping_memberof_attribute']);
					$statusMessage='';
					if(!get_option('mo_ldap_enable_role_mapping'))
						$statusMessage = ' Please check <b>"Enable Role Mapping"</b> to activate it.';
					update_option( 'mo_ldap_message', 'LDAP Group to WP role mapping has been updated.'.$statusMessage);
					$this->show_success_message();
				} else if($_POST['option'] == 'mo_ldap_cloud_save_attribute_config'){
					
					$custom_attributes = array();
					foreach($_POST as $key=>$value){
						if(strpos($key, "mo_ldap_custom_attribute") === false){

						} else{
							array_push($custom_attributes, $key);
						}
							
					}
					update_option( 'mo_ldap_enable_attribute_mapping', isset($_POST['enable_update_ldap']) ? $_POST['enable_update_ldap'] : 0);

					$email_attribute = sanitize_text_field($_POST['mo_ldap_email_attribute']);
					$phone_attribute = sanitize_text_field($_POST['mo_ldap_phone_attribute']);
					$fname_attribute = sanitize_text_field($_POST['mo_ldap_fname_attribute']);
					$lname_attribute = sanitize_text_field($_POST['mo_ldap_lname_attribute']);
					//$enabled_attribute = sanitize_text_field($_POST['mo_ldap_user_enabled_attribute']);

					update_option('mo_ldap_email_attribute', $email_attribute);
					update_option('mo_ldap_phone_attribute', $phone_attribute);
					update_option('mo_ldap_fname_attribute', $fname_attribute);
					update_option('mo_ldap_lname_attribute', $lname_attribute);
					
					foreach($custom_attributes as $attribute){
						if(isset($_POST[$attribute]) and $_POST[$attribute] != ''){
							if(get_option($attribute) == null){
								$attribute_key = 'mo_ldap_custom_attribute_' . strtolower(sanitize_text_field($_POST[$attribute]));
							} else{
								$attribute_key = strtolower(sanitize_text_field($_POST[$attribute]));
							}
							update_option($attribute_key, strtolower(sanitize_text_field($_POST[$attribute])));
							
						}
					}
					
					update_option( 'mo_ldap_message', 'Successfully saved LDAP Attribute Configuration');
					$this->show_success_message();

				} else if($_POST['option'] == 'mo_ldap_delete_custom_attribute'){
					$custom_attribute_name = sanitize_text_field($_POST['custom_attribute_name']);
					$custom_attribute_key = 'mo_ldap_custom_attribute_' . $custom_attribute_name;
					delete_option($custom_attribute_key);
					update_option('mo_ldap_message', 'Successfully deleted custom attribute: <b>' . $custom_attribute_name . '</b>');
					$this->show_success_message();
				} else if(isset($_POST['option']) and trim($_POST['option']) == "mo_ldap_check_license"){	
					$this->mo_ldap_handle_mo_check_ln(true);
				} else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_ldap_go_back" ){
					update_option('mo_ldap_registration_status','');
					update_option('mo_ldap_verify_customer', '');
					delete_option('mo_ldap_new_registration');
					delete_option('mo_ldap_admin_email');
					delete_option('mo_ldap_admin_phone');
				}else if(isset($_POST['option']) && $_POST['option'] == 'mo_ldap_forgot_password_form_option'){
					
					$email = get_option('mo_ldap_admin_email');
					$customer = new Mo_Ldap_Customer();
					$content = json_decode($customer->mo_ldap_forgot_password($email),true);
					if(strcasecmp($content['status'], 'SUCCESS') == 0){
						update_option( 'mo_ldap_message','Your password has been reset successfully. Please enter the new password sent to ' . $email . '.');
						$this->show_success_message();
					}else{
						update_option( 'mo_ldap_message','An error occured while processing your request. Please Try again.');
						$this->show_error_message();
					}
				}
			}
		}
		
		
		function test_attribute_configuration(){
			if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'testcloudrolemappingconfig'){
				$username = $_REQUEST['user'];
				$mo_ldap_role_mapping = new Mo_Ldap_Role_Mapping();
				$mo_ldap_role_mapping->test_configuration($username);
			} else if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'testldapattrconfig'){
				$username = $_REQUEST['user'];
				$mo_ldap_config = new Mo_Ldap_Config();
				$mo_ldap_config->test_attribute_configuration($username);
			}
		}
		
		
		/*
		 * Save all required fields on customer registration/retrieval complete.
		 */
		function save_success_customer_config($id, $apiKey, $token, $message) {
			update_option( 'mo_ldap_admin_customer_key', $id );
			update_option( 'mo_ldap_admin_api_key', $apiKey );
			update_option( 'mo_ldap_customer_token', $token );
			update_option('mo_ldap_password', '');
			update_option( 'mo_ldap_message', $message);
			delete_option('mo_ldap_verify_customer');
			delete_option('mo_ldap_new_registration');
			delete_option('mo_ldap_registration_status');
			
			$this->show_success_message();
			if(strcmp($message,'Registration complete!') == 0 ){
				wp_redirect(admin_url().'admin.php?page=mo_ldap_login&tab=pricing');
			}
		}
		
		function mo_ldap_settings_style() {
			wp_enqueue_style( 'mo_ldap_admin_settings_style', plugins_url('includes/css/style_settings.css', __FILE__));
			wp_enqueue_style( 'mo_ldap_admin_settings_phone_style', plugins_url('includes/css/phone.css', __FILE__));
		}

		function mo_ldap_settings_script() {
			wp_enqueue_script( 'mo_ldap_admin_settings_phone_script', plugins_url('includes/js/phone.js', __FILE__ ));
			wp_enqueue_script( 'mo_ldap_admin_settings_script', plugins_url('includes/js/settings_page.js', __FILE__ ), array('jquery'));
		}
		
		function success_message() {
			$class = "error";
			$message = get_option('mo_ldap_message');
			echo "<div class='" . $class . "'> <p>" . $message . "</p></div>"; 
		}
		
		function error_message() {
			$class = "updated";
			$message = get_option('mo_ldap_message');
			echo "<div class='" . $class . "'> <p>" . $message . "</p></div>"; 
		}
		
		private function show_success_message() {
			remove_action( 'admin_notices', array( $this, 'success_message') );
			add_action( 'admin_notices', array( $this, 'error_message') );
		}
		
		private function show_error_message() {
			remove_action( 'admin_notices', array( $this, 'error_message') );
			add_action( 'admin_notices', array( $this, 'success_message') );
		}
		
		function plugin_query_vars($vars) {
			$vars[] = 'app_name';
			return $vars;
		}
		
		function parse_sso_request($wp){
			if (array_key_exists('app_name', $wp->query_vars)){
				$redirectUrl = mo_ldap_saml_login($wp->query_vars['app_name']);
				wp_redirect($redirectUrl, 302);
				exit;
			}
		}
		
		public function mo_ldap_activate() {
			update_option( 'mo_ldap_register_user',1);
			update_option('mo_ldap_xecurify_migration', 'true');
		}
		
		public function mo_ldap_deactivate() {
			//delete all stored key-value pairs
			if( !Mo_Ldap_Util::check_empty_or_null( get_option('mo_ldap_registration_status') ) ) {
				delete_option('mo_ldap_admin_email');
			}
			
			delete_option('mo_ldap_host_name');
			delete_option('mo_ldap_default_config');
			delete_option('mo_ldap_password');
			delete_option('mo_ldap_new_registration');
			delete_option('mo_ldap_admin_phone');
			delete_option('mo_ldap_verify_customer');
			delete_option('mo_ldap_admin_customer_key');
			delete_option('mo_ldap_admin_api_key');
			delete_option('mo_ldap_customer_token');
			delete_option('mo_ldap_message');
			
			delete_option('mo_ldap_enable_login');
			delete_option('mo_ldap_server_password');
			
			delete_option('mo_ldap_transactionId');
			delete_option('mo_ldap_registration_status');

			
		}
		
		public function update_user_attributes($profileAttributes , $userid){
			
			$email_attribute = get_option('mo_ldap_email_attribute');
			$phone_attribute = get_option('mo_ldap_phone_attribute');
			$fname_attribute = get_option('mo_ldap_fname_attribute');
			$lname_attribute = get_option('mo_ldap_lname_attribute');
			
			$firstName = $profileAttributes->$fname_attribute;
			$lastName = $profileAttributes->$lname_attribute;
			$email = $profileAttributes->$email_attribute;
			$phone = $profileAttributes->$phone_attribute;
			
			  //Update email, phone, fname and lname attributes for userssssss
			  $user_data['ID'] = $userid;
			  if($email && $email!="")
				$user_data['user_email'] = $email;
			  if($firstName && $firstName!="")
				  $user_data['first_name'] = $firstName;
			  if($lastName && $lastName!="")
				$user_data['last_name'] = $lastName;
			wp_update_user($user_data);
			
			
			$wp_options = wp_load_alloptions();
			foreach($wp_options as $option=>$value){
				if(strpos($option, "mo_ldap_custom_attribute") === false){
					//Do nothing
				} else{
					$avalue = $profileAttributes->$value;
					update_user_meta($userid, $option, $avalue);
				}
			}

			//exit();
		}
		
		private function mo_ldap_handle_mo_check_ln($showMessage){
		$challenge_otp = new MO_Ldap_Util();
		$content = json_decode($challenge_otp->check_customer_ln(), true);
		if(strcasecmp($content['status'], 'SUCCESS') == 0){
			array_key_exists('licensePlan', $content) && !Mo_Ldap_Util::check_empty_or_null($content['licensePlan']) ? update_option('mo_ldap_check_ln',base64_encode($content['licensePlan'])) : update_option('mo_customer_check_ln','');
			if($showMessage)
				array_key_exists('licensePlan', $content) && !Mo_Ldap_Util::check_empty_or_null($content['licensePlan']) ? update_option('mo_ldap_message', 'Thank you. You have upgraded to '.$content['licensePlan']) : update_option('mo_ldap_message', 'You are on our FREE plan. Check Licensing Tab to learn how to upgrade.');
			if(array_key_exists('licensePlan', $content) && !Mo_Ldap_Util::check_empty_or_null($content['licensePlan'])){
				delete_option('mo_ldap_free_version');
			}
			$this->show_success_message();
		}else{
			update_option('mo_ldap_message', 'You are on our FREE plan. Check Licensing Tab to learn how to upgrade.');
			$this->show_success_message();
		}
	}
		
	}
	
	new Mo_Ldap_Login;
?>